==> theia-0.1.8.12-py3-none-any.whl: theia/aws/network/route53.py <==
import pulumi
from pulumi import Input, InputType

from pulumi_aws import route53

from typing import Optional, Mapping, Literal, List, Any
from pydantic.dataclasses import dataclass, ConfigDict
from theia.utils.errors import TheiaConfigurationError

from theia import ResourceOptions
from theia._registry import StackRegistry, GlobalTags
from theia.definitions import THEIA_VERSION
from theia.aws.network.eip import Eip
from pulumi import Output
import time
import re


class AliasZoneID:
    application_and_classic_zone_ids = {
        "us-east-2": "Z3AADJGX6KTTL2",
        "us-east-1": "Z35SXDOTRQ7X7K",
        "us-west-1": "Z368ELLRRE2KJ0",
        "us-west-2": "Z1H1FL5HABSF5",
        "af-south-1": "Z268VQBMOI5EKX",
        "ap-east-1": "Z3DQVH9N71FHZ0",
        "ap-south-2": "Z0173938T07WNTVAEPZN",
        "ap-southeast-3": "Z08888821HLRG5A9ZRTER",
        "ap-southeast-4": "Z09517862IB2WZLPXG76F",
        "ap-south-1": "ZP97RAFLXTNZK",
        "ap-northeast-3": "Z5LXEXXYW11ES",
        "ap-northeast-2": "ZWKZPGTI48KDX",
        "ap-southeast-1": "Z1LMS91P8CMLE5",
        "ap-southeast-2": "Z1GM3OXH4ZPM65",
        "ap-northeast-1": "Z14GRHDCWA56QT",
        "ca-central-1": "ZQSVJUPU6J1EY",
        "cn-north-1": "Z1GDH35T77C1KE",
        "cn-northwest-1": "ZM7IZAIOVVDZF",
        "eu-central-1": "Z215JYRZR1TBD5",
        "eu-west-1": "Z32O12XQLNTSW2",
        "eu-west-2": "ZHURV8PSTC4K8",
        "eu-south-1": "Z3ULH7SSC9OV64",
        "eu-west-3": "Z3Q77PNBQS71R4",
        "eu-south-2": "Z0956581394HF5D5LXGAP",
        "eu-north-1": "Z23TAZ6LKFMNIO",
        "eu-central-2": "Z06391101F2ZOEP8P5EB3",
        "il-central-1": "Z09170902867EHPV2DABU",
        "me-south-1": "ZS929ML54UICD",
        "me-central-1": "Z08230872XQRWHG2XF6I",
        "sa-east-1": "Z2P70J7HTTTPLU",
        "us-gov-east-1": "Z166TLBEWOO7G0",
        "us-gov-west-1": "Z33AYJ8TM3BH4J",
    }

    network_zone_ids = {
        "us-east-2": "ZLMOA37VPKANP",
        "us-east-1": "Z26RNL4JYFTOTI",
        "us-west-1": "Z24FKFUX50B4VW",
        "us-west-2": "Z18D5FSROUN65G",
        "af-south-1": "Z203XCE67M25HM",
        "ap-east-1": "Z12Y7K3UBGUAD1",
        "ap-south-2": "Z0711778386UTO08407HT",
        "ap-southeast-3": "Z01971771FYVNCOVWJU1G",
        "ap-southeast-4": "Z01156963G8MIIL7X90IV",
        "ap-south-1": "ZVDDRBQ08TROA",
        "ap-northeast-3": "Z1GWIQ4HH19I5X",
        "ap-northeast-2": "ZIBE1TIR4HY56",
        "ap-southeast-1": "ZKVM4W9LS7TM",
        "ap-southeast-2": "ZCT6FZBF4DROD",
        "ap-northeast-1": "Z31USIVHYNEOWT",
        "ca-central-1": "Z2EPGBW3API2WT",
        "cn-north-1": "Z3QFB96KMJ7ED6",
        "cn-northwest-1": "ZQEIKTCZ8352D",
        "eu-central-1": "Z3F0SRJ5LGBH90",
        "eu-west-1": "Z2IFOLAFXWLO4F",
        "eu-west-2": "ZD4D7Y8KGAS4G",
        "eu-south-1": "Z23146JA1KNAFP",
        "eu-west-3": "Z1CMS0P5QUZ6D5",
        "eu-south-2": "Z1011216NVTVYADP1SSV",
        "eu-north-1": "Z1UDT6IFJ4EJM",
        "eu-central-2": "Z02239872DOALSIDCX66S",
        "il-central-1": "Z0313266YDI6ZRHTGQY4",
        "me-south-1": "Z3QSRYVP46NYYV",
        "me-central-1": "Z00282643NTTLPANJJG2P",
        "sa-east-1": "ZTK26PT1VY4CU",
        "us-gov-east-1": "Z1ZSMQQ6Q24QQ8",
        "us-gov-west-1": "ZMG1MZ2THAWF1",
    }

    @staticmethod
    def get_zone_id(alias_destination_ep: str) -> str:

        regex_application_and_classic_loadbalancer_dns = ".+\.(.*)\.elb\.amazonaws\.com"
        regex_network_loadbalancer_dns = ".+\.elb\.(.*)\.amazonaws\.com"
        regex_cloufront_distribution_domain_name = ".*\.cloudfront\.net"

        if re.match(regex_application_and_classic_loadbalancer_dns, alias_destination_ep):
            region = re.search(regex_application_and_classic_loadbalancer_dns, alias_destination_ep).group(1)
            return AliasZoneID.application_and_classic_zone_ids[region]

        elif re.match(regex_network_loadbalancer_dns, alias_destination_ep):
            region = re.search(regex_network_loadbalancer_dns, alias_destination_ep).group(1)
            return AliasZoneID.network_zone_ids[region]

        elif re.match(regex_cloufront_distribution_domain_name, alias_destination_ep):
            return "Z2FDTNDATAQYW2"
        else:
            return None


@dataclass
class RecordAliasArgs:
    destination_ep: str
    evaluate_target_health: Optional[bool] = False

    def __post_init__(self):

        self.zone_id = AliasZoneID.get_zone_id(
            alias_destination_ep=self.destination_ep,
        )

        if not self.zone_id:
            raise TheiaConfigurationError("destination_ep is not valid")


@dataclass(config=ConfigDict(arbitrary_types_allowed=True))
class ZoneVpcArgs:
    vpc_id: str | Output[str]
    vpc_region: Optional[str] = None


@dataclass
class RecordCidrRoutingPolicyArgs:
    collection_id: str
    location_name: str


@dataclass
class RecordGeolocationRoutingPolicyArgs:
    continent: str
    country: str
    subdivision: str


@dataclass(config=ConfigDict(arbitrary_types_allowed=True))
class RecordArgs:
    hosted_zone_name: str
    type: (
        Literal["A", "AAAA", "CNAME", "CAA", "DS", "MX", "NAPTR", "NS", "PTR", "SOA", "SPF", "SRV", "TXT", "ALIAS"]
        | None
    ) = None
    zone_id: Output[str] = None
    aliases: Optional[List[RecordAliasArgs]] = None
    allow_overwrite: Optional[bool] = None
    cidr_routing_policy: Optional[RecordCidrRoutingPolicyArgs] = None
    failover_routing_policies: Optional[List[str]] = None
    geolocation_routing_policies: Optional[List[RecordGeolocationRoutingPolicyArgs]] = None
    health_check_id: Optional[str] = None
    latency_routing_policies: List[str] = None
    multivalue_answer_routing_policy: Optional[bool] = None
    records: List[str] | Output[List[str]] | List[Output[str]] = None
    set_identifier: Optional[str] = None
    ttl: Optional[int] = None
    weighted_routing_policies: Optional[List[int]] = None

    def __post_init__(self):
        if self.aliases and (self.records or self.ttl):
            raise TheiaConfigurationError("Cannot set records and ttl while creating alias")

        if not self.aliases and not self.records:
            raise TheiaConfigurationError(
                "Exactly one of 'records' or 'aliases' must be specified: this determines whether it's an alias record"
            )

        if not self.aliases and not self.type:
            raise TheiaConfigurationError("type is required while creating non-alias record")

        if self.aliases and not self.type:
            self.type = "A"


class Record:
    def __new__(
        cls, name: str, record_args: RecordArgs, opts: Optional[ResourceOptions] = None, __props__: Any | None = None
    ) -> route53.Record:

        alias_args = None
        if record_args.aliases:
            alias_args = []
            for alias_record in record_args.aliases:
                alias_args.append(
                    route53.RecordAliasArgs(
                        name=alias_record.destination_ep,
                        zone_id=alias_record.zone_id,
                        evaluate_target_health=alias_record.evaluate_target_health,
                    )
                )

        if not record_args.zone_id:
            record_args.zone_id = route53.get_zone(name=record_args.hosted_zone_name).id

        record_args_value = route53.RecordArgs(
            name=name,
            type=record_args.type,
            zone_id=record_args.zone_id,
            aliases=alias_args,
            allow_overwrite=record_args.allow_overwrite,
            cidr_routing_policy=record_args.cidr_routing_policy,
            failover_routing_policies=record_args.failover_routing_policies,
            geolocation_routing_policies=record_args.geolocation_routing_policies,
            health_check_id=record_args.health_check_id,
            latency_routing_policies=record_args.latency_routing_policies,
            multivalue_answer_routing_policy=record_args.multivalue_answer_routing_policy,
            records=record_args.records,
            set_identifier=record_args.set_identifier,
            ttl=record_args.ttl,
            weighted_routing_policies=record_args.weighted_routing_policies,
        )

        resource_name = name + "-" + record_args.type + "-" + record_args.hosted_zone_name

        print(resource_name)

        record = route53.Record(
            resource_name,
            args=record_args_value,
            opts=opts,
        )

        pulumi.export(
            "Route53-Record---" + resource_name,
            {
                "id": record.id,
                "fqdn": record.fqdn,
            },
        )

        return record


class Zone:
    def __new__(
        cls,
        resource_name: str,
        comment: Optional[str] = None,
        tags: Optional[Mapping[str, str]] = None,
        vpcs: Optional[List[ZoneVpcArgs]] = None,
        delegation_set_id: Optional[str] = None,
        opts: Optional[ResourceOptions] = None,
    ) -> route53.Zone:

        vpcs_args = None
        if vpcs:
            vpcs_args = []
            for vpcs_arg in vpcs:
                vpcs_args.append(
                    route53.ZoneVpcArgs(
                        vpc_id=vpcs_arg.vpc_id,
                        vpc_region=vpcs_arg.vpc_region,
                    )
                )

        hosted_zone = route53.Zone(
            resource_name,
            name=resource_name,
            vpcs=vpcs_args,
            opts=opts,
            comment=comment,
            tags=tags,
            delegation_set_id=delegation_set_id,
        )

        pulumi.export(
            "Route53-Hosted_Zone---" + resource_name,
            {
                "arn": hosted_zone.arn,
                "id": hosted_zone.id,
                "name_servers": hosted_zone.name_servers,
                "zone": hosted_zone.zone_id,
            },
        )

        return hosted_zone


@dataclass(config=ConfigDict(arbitrary_types_allowed=True))
class ResolverRuleIps:
    ip: str | Output[str]
    port: Optional[int] = None


@dataclass
class ResolverRuleEndpointIP:
    subnet_id: str
    ip: Optional[str] = None


@dataclass(config=ConfigDict(arbitrary_types_allowed=True))
class ResolverRuleEndpoint:
    ip_addresses: List[ResolverRuleEndpointIP]
    direction: Literal["INBOUND", "OUTBOUND"] = "INBOUND"
    protocols: Optional[List[Literal["DoH", "Do53", "DoH-FIPS"]]] = None
    resolver_endpoint_type: Optional[Literal["IPV4", "IPV6", "DUALSTACK"]] = None
    security_group_ids: Optional[List[str | Output[str]]] = None
    tags: Optional[Mapping[str, str]] = None


@dataclass
class ResolverRuleSettings:
    vpc_id: Optional[str] = None
    domain_name: Optional[str] = None
    outbount_endpoint: Optional[ResolverRuleEndpoint] = None
    rule_type: Optional[Literal["FORWARD", "SYSTEM", "RECURSIVE"]] = "FORWARD"
    resolver_endpoint_id: Optional[str] = None
    target_ips: Optional[List[ResolverRuleIps]] = None


class ResolverRule:
    def __new__(
        cls,
        resource_name: str,
        settings: Optional[ResolverRuleSettings] = None,
        tags: Optional[Mapping[str, str]] = None,
        opts: Optional[ResourceOptions] = None,
    ):

        name = {"Name": resource_name}
        merged_tags = THEIA_VERSION | GlobalTags.get() | name
        if tags:
            merged_tags = tags | merged_tags

        if settings.outbount_endpoint:
            endpoint = route53.ResolverEndpoint(
                resource_name=f"{resource_name}-outbound-endpoint",
                name=f"{resource_name}-outbound-endpoint",
                direction=settings.outbount_endpoint.direction,
                security_group_ids=settings.outbount_endpoint.security_group_ids,
                ip_addresses=[
                    route53.ResolverEndpointIpAddressArgs(
                        subnet_id=ip_add.subnet_id,
                        ip=ip_add.ip,
                    )
                    for ip_add in settings.outbount_endpoint.ip_addresses
                ],
                protocols=settings.outbount_endpoint.protocols,
                tags=merged_tags,
            )

        if settings.target_ips:
            target_ips = [
                route53.ResolverRuleTargetIpArgs(ip=target.ip, port=target.port) for target in settings.target_ips
            ]

        resolver_rule = route53.ResolverRule(
            resource_name=resource_name,
            name=resource_name,
            domain_name=settings.domain_name,
            rule_type=settings.rule_type,
            resolver_endpoint_id=None if not settings.outbount_endpoint else endpoint.id,
            target_ips=target_ips,
            opts=opts,
        )

        if settings.vpc_id:
            route53.ResolverRuleAssociation(
                resource_name=f"{resource_name}-assc", vpc_id=settings.vpc_id, resolver_rule_id=resolver_rule.id
            )

        pulumi.export(
            "Route53-Resolver---" + resource_name,
            {
                "arn": resolver_rule.arn,
                "id": resolver_rule.id,
            },
        )

        return resolver_rule

==> theia-0.1.8.24-py3-none-any.whl: theia/aws/network/route53.py <==
import pulumi
from pulumi import Input, InputType

from pulumi_aws import route53

from typing import Optional, Mapping, Literal, List, Any
from pydantic.dataclasses import dataclass, ConfigDict
from theia.utils.errors import TheiaConfigurationError

from theia import ResourceOptions
from theia._registry import StackRegistry, GlobalTags
from theia.definitions import THEIA_VERSION
from theia.aws.network.eip import Eip
from pulumi import Output
import time
import re


class AliasZoneID:
    application_and_classic_zone_ids = {
        "us-east-2": "Z3AADJGX6KTTL2",
        "us-east-1": "Z35SXDOTRQ7X7K",
        "us-west-1": "Z368ELLRRE2KJ0",
        "us-west-2": "Z1H1FL5HABSF5",
        "af-south-1": "Z268VQBMOI5EKX",
        "ap-east-1": "Z3DQVH9N71FHZ0",
        "ap-south-2": "Z0173938T07WNTVAEPZN",
        "ap-southeast-3": "Z08888821HLRG5A9ZRTER",
        "ap-southeast-4": "Z09517862IB2WZLPXG76F",
        "ap-south-1": "ZP97RAFLXTNZK",
        "ap-northeast-3": "Z5LXEXXYW11ES",
        "ap-northeast-2": "ZWKZPGTI48KDX",
        "ap-southeast-1": "Z1LMS91P8CMLE5",
        "ap-southeast-2": "Z1GM3OXH4ZPM65",
        "ap-northeast-1": "Z14GRHDCWA56QT",
        "ca-central-1": "ZQSVJUPU6J1EY",
        "cn-north-1": "Z1GDH35T77C1KE",
        "cn-northwest-1": "ZM7IZAIOVVDZF",
        "eu-central-1": "Z215JYRZR1TBD5",
        "eu-west-1": "Z32O12XQLNTSW2",
        "eu-west-2": "ZHURV8PSTC4K8",
        "eu-south-1": "Z3ULH7SSC9OV64",
        "eu-west-3": "Z3Q77PNBQS71R4",
        "eu-south-2": "Z0956581394HF5D5LXGAP",
        "eu-north-1": "Z23TAZ6LKFMNIO",
        "eu-central-2": "Z06391101F2ZOEP8P5EB3",
        "il-central-1": "Z09170902867EHPV2DABU",
        "me-south-1": "ZS929ML54UICD",
        "me-central-1": "Z08230872XQRWHG2XF6I",
        "sa-east-1": "Z2P70J7HTTTPLU",
        "us-gov-east-1": "Z166TLBEWOO7G0",
        "us-gov-west-1": "Z33AYJ8TM3BH4J",
    }

    network_zone_ids = {
        "us-east-2": "ZLMOA37VPKANP",
        "us-east-1": "Z26RNL4JYFTOTI",
        "us-west-1": "Z24FKFUX50B4VW",
        "us-west-2": "Z18D5FSROUN65G",
        "af-south-1": "Z203XCE67M25HM",
        "ap-east-1": "Z12Y7K3UBGUAD1",
        "ap-south-2": "Z0711778386UTO08407HT",
        "ap-southeast-3": "Z01971771FYVNCOVWJU1G",
        "ap-southeast-4": "Z01156963G8MIIL7X90IV",
        "ap-south-1": "ZVDDRBQ08TROA",
        "ap-northeast-3": "Z1GWIQ4HH19I5X",
        "ap-northeast-2": "ZIBE1TIR4HY56",
        "ap-southeast-1": "ZKVM4W9LS7TM",
        "ap-southeast-2": "ZCT6FZBF4DROD",
        "ap-northeast-1": "Z31USIVHYNEOWT",
        "ca-central-1": "Z2EPGBW3API2WT",
        "cn-north-1": "Z3QFB96KMJ7ED6",
        "cn-northwest-1": "ZQEIKTCZ8352D",
        "eu-central-1": "Z3F0SRJ5LGBH90",
        "eu-west-1": "Z2IFOLAFXWLO4F",
        "eu-west-2": "ZD4D7Y8KGAS4G",
        "eu-south-1": "Z23146JA1KNAFP",
        "eu-west-3": "Z1CMS0P5QUZ6D5",
        "eu-south-2": "Z1011216NVTVYADP1SSV",
        "eu-north-1": "Z1UDT6IFJ4EJM",
        "eu-central-2": "Z02239872DOALSIDCX66S",
        "il-central-1": "Z0313266YDI6ZRHTGQY4",
        "me-south-1": "Z3QSRYVP46NYYV",
        "me-central-1": "Z00282643NTTLPANJJG2P",
        "sa-east-1": "ZTK26PT1VY4CU",
        "us-gov-east-1": "Z1ZSMQQ6Q24QQ8",
        "us-gov-west-1": "ZMG1MZ2THAWF1",
    }

    @staticmethod
    def get_zone_id(alias_destination_ep: str) -> str:

        regex_application_and_classic_loadbalancer_dns = ".+\.(.*)\.elb\.amazonaws\.com"
        regex_network_loadbalancer_dns = ".+\.elb\.(.*)\.amazonaws\.com"
        regex_cloufront_distribution_domain_name = ".*\.cloudfront\.net"

        if re.match(regex_application_and_classic_loadbalancer_dns, alias_destination_ep):
            region = re.search(regex_application_and_classic_loadbalancer_dns, alias_destination_ep).group(1)
            return AliasZoneID.application_and_classic_zone_ids[region]

        elif re.match(regex_network_loadbalancer_dns, alias_destination_ep):
            region = re.search(regex_network_loadbalancer_dns, alias_destination_ep).group(1)
            return AliasZoneID.network_zone_ids[region]

        elif re.match(regex_cloufront_distribution_domain_name, alias_destination_ep):
            return "Z2FDTNDATAQYW2"
        else:
            return None


@dataclass
class RecordAliasArgs:
    destination_ep: str
    evaluate_target_health: Optional[bool] = False

    def __post_init__(self):

        self.zone_id = AliasZoneID.get_zone_id(
            alias_destination_ep=self.destination_ep,
        )

        if not self.zone_id:
            raise TheiaConfigurationError("destination_ep is not valid")


@dataclass(config=ConfigDict(arbitrary_types_allowed=True))
class ZoneVpcArgs:
    vpc_id: str | Output[str]
    vpc_region: Optional[str] = None


@dataclass
class RecordCidrRoutingPolicyArgs:
    collection_id: str
    location_name: str


@dataclass
class RecordGeolocationRoutingPolicyArgs:
    continent: str
    country: str
    subdivision: str


@dataclass(config=ConfigDict(arbitrary_types_allowed=True))
class RecordArgs:
    name: str
    hosted_zone_name: Optional[str] = None
    type: (
        Literal["A", "AAAA", "CNAME", "CAA", "DS", "MX", "NAPTR", "NS", "PTR", "SOA", "SPF", "SRV", "TXT", "ALIAS"]
        | None
    ) = None
    zone_id: Output[str] | str = None
    aliases: Optional[List[RecordAliasArgs]] = None
    allow_overwrite: Optional[bool] = None
    cidr_routing_policy: Optional[RecordCidrRoutingPolicyArgs] = None
    failover_routing_policies: Optional[List[str]] = None
    geolocation_routing_policies: Optional[List[RecordGeolocationRoutingPolicyArgs]] = None
    health_check_id: Optional[str] = None
    latency_routing_policies: List[str] = None
    multivalue_answer_routing_policy: Optional[bool] = None
    records: List[str] | Output[List[str]] | List[Output[str]] = None
    set_identifier: Optional[str] = None
    ttl: Optional[int] = None
    weighted_routing_policies: Optional[List[int]] = None

    def __post_init__(self):
        if self.aliases and (self.records or self.ttl):
            raise TheiaConfigurationError("Cannot set records and ttl while creating alias")

        if not self.aliases and not self.records:
            raise TheiaConfigurationError(
                "Exactly one of 'records' or 'aliases' must be specified: this determines whether it's an alias record"
            )

        if not self.aliases and not self.type:
            raise TheiaConfigurationError("type is required while creating non-alias record")

        if self.aliases and not self.type:
            self.type = "A"

        if self.records and not self.ttl:
            self.ttl = 300


class Record:
    def __new__(
        cls,
        resource_name: str,
        record_args: RecordArgs,
        opts: Optional[ResourceOptions] = None,
        __props__: Any | None = None,
    ) -> route53.Record:

        alias_args = None
        if record_args.aliases:
            alias_args = []
            for alias_record in record_args.aliases:
                alias_args.append(
                    route53.RecordAliasArgs(
                        name=alias_record.destination_ep,
                        zone_id=alias_record.zone_id,
                        evaluate_target_health=alias_record.evaluate_target_health,
                    )
                )

        if not record_args.zone_id:
            record_args.zone_id = route53.get_zone(name=record_args.hosted_zone_name).id

        record_args_value = route53.RecordArgs(
            name=record_args.name,
            type=record_args.type,
            zone_id=record_args.zone_id,
            aliases=alias_args,
            allow_overwrite=record_args.allow_overwrite,
            cidr_routing_policy=record_args.cidr_routing_policy,
            failover_routing_policies=record_args.failover_routing_policies,
            geolocation_routing_policies=record_args.geolocation_routing_policies,
            health_check_id=record_args.health_check_id,
            latency_routing_policies=record_args.latency_routing_policies,
            multivalue_answer_routing_policy=record_args.multivalue_answer_routing_policy,
            records=record_args.records,
            set_identifier=record_args.set_identifier,
            ttl=record_args.ttl,
            weighted_routing_policies=record_args.weighted_routing_policies,
        )

        record = route53.Record(
            resource_name=resource_name,
            args=record_args_value,
            opts=opts,
        )

        pulumi.export(
            "Route53-Record---" + resource_name,
            {
                "id": record.id,
                "fqdn": record.fqdn,
            },
        )

        return record


class Zone:
    def __new__(
        cls,
        resource_name: str,
        comment: Optional[str] = None,
        tags: Optional[Mapping[str, str]] = None,
        vpcs: Optional[List[ZoneVpcArgs]] = None,
        delegation_set_id: Optional[str] = None,
        opts: Optional[ResourceOptions] = None,
    ) -> route53.Zone:

        vpcs_args = None
        if vpcs:
            vpcs_args = []
            for vpcs_arg in vpcs:
                vpcs_args.append(
                    route53.ZoneVpcArgs(
                        vpc_id=vpcs_arg.vpc_id,
                        vpc_region=vpcs_arg.vpc_region,
                    )
                )

        hosted_zone = route53.Zone(
            resource_name,
            name=resource_name,
            vpcs=vpcs_args,
            opts=opts,
            comment=comment,
            tags=tags,
            delegation_set_id=delegation_set_id,
        )

        pulumi.export(
            "Route53-Hosted_Zone---" + resource_name,
            {
                "arn": hosted_zone.arn,
                "id": hosted_zone.id,
                "name_servers": hosted_zone.name_servers,
                "zone": hosted_zone.zone_id,
            },
        )

        return hosted_zone


@dataclass(config=ConfigDict(arbitrary_types_allowed=True))
class ResolverRuleIps:
    ip: str | Output[str]
    port: Optional[int] = None


@dataclass
class ResolverRuleEndpointIP:
    subnet_id: str
    ip: Optional[str] = None


@dataclass(config=ConfigDict(arbitrary_types_allowed=True))
class ResolverRuleEndpoint:
    ip_addresses: List[ResolverRuleEndpointIP]
    direction: Literal["INBOUND", "OUTBOUND"] = "INBOUND"
    protocols: Optional[List[Literal["DoH", "Do53", "DoH-FIPS"]]] = None
    resolver_endpoint_type: Optional[Literal["IPV4", "IPV6", "DUALSTACK"]] = None
    security_group_ids: Optional[List[str | Output[str]]] = None
    tags: Optional[Mapping[str, str]] = None


@dataclass
class ResolverRuleSettings:
    vpc_id: Optional[str] = None
    domain_name: Optional[str] = None
    outbount_endpoint: Optional[ResolverRuleEndpoint] = None
    rule_type: Optional[Literal["FORWARD", "SYSTEM", "RECURSIVE"]] = "FORWARD"
    resolver_endpoint_id: Optional[str] = None
    target_ips: Optional[List[ResolverRuleIps]] = None


class ResolverRule:
    def __new__(
        cls,
        resource_name: str,
        settings: Optional[ResolverRuleSettings] = None,
        tags: Optional[Mapping[str, str]] = None,
        opts: Optional[ResourceOptions] = None,
    ):

        name = {"Name": resource_name}
        merged_tags = THEIA_VERSION | GlobalTags.get() | name
        if tags:
            merged_tags = tags | merged_tags

        if settings.outbount_endpoint:
            endpoint = route53.ResolverEndpoint(
                resource_name=f"{resource_name}-outbound-endpoint",
                name=f"{resource_name}-outbound-endpoint",
                direction=settings.outbount_endpoint.direction,
                security_group_ids=settings.outbount_endpoint.security_group_ids,
                ip_addresses=[
                    route53.ResolverEndpointIpAddressArgs(
                        subnet_id=ip_add.subnet_id,
                        ip=ip_add.ip,
                    )
                    for ip_add in settings.outbount_endpoint.ip_addresses
                ],
                protocols=settings.outbount_endpoint.protocols,
                tags=merged_tags,
            )

        if settings.target_ips:
            target_ips = [
                route53.ResolverRuleTargetIpArgs(ip=target.ip, port=target.port) for target in settings.target_ips
            ]

        resolver_rule = route53.ResolverRule(
            resource_name=resource_name,
            name=resource_name,
            domain_name=settings.domain_name,
            rule_type=settings.rule_type,
            resolver_endpoint_id=None if not settings.outbount_endpoint else endpoint.id,
            target_ips=target_ips,
            opts=opts,
        )

        if settings.vpc_id:
            route53.ResolverRuleAssociation(
                resource_name=f"{resource_name}-assc", vpc_id=settings.vpc_id, resolver_rule_id=resolver_rule.id
            )

        pulumi.export(
            "Route53-Resolver---" + resource_name,
            {
                "arn": resolver_rule.arn,
                "id": resolver_rule.id,
            },
        )

        return resolver_rule
